# Creates a feature set based on the input features given
# 
# Args:
#  inp.set : Input features. Can be a list of feature names or feature ids from
#	     the relational table.
#  dataset.id : The dataset to which the features belong to.

AssignFeatureSet <- function(inp.set, dataset.id) {


	
	
	# Get the feature Ids (ints) if the input set is represented
	# in terms of feature names	
	if(is.character(inp.set) == TRUE) {
		inp.set <- GetFeatureIds(inp.set, dataset.id)
	}

	# Assign feature set is a special case, where there is no 
	# distinction between interactive or batching.
	
	result	<- ExeAssignFeatureSet(inp.set, dataset.id)

	if(is.updateprogram()) {
		pgm.str <- paste("fs", result$out.set$feat.id, " <- ", "AssignFeatureSet(inp.set = c(", paste(inp.set, collapse=","), "), dataset.id = ", dataset.id, ")", sep="")
		AddProgramLine(pgm.str)
	}
	
	out.set <- result$out.set
	
}


AddFeatureSet <- function(inp.set1, inp.set2, dataset.id) {
	
	# Error handling.
	if(length(inp.set1) == 0) {
		stop("Argument inp.set1 cannot be of length 0")
	}
	
	if(length(inp.set2) == 0) {
		stop("Argument inp.set2 cannot be of length 0")
	}
	
	result <- SkelAddFeatureSet(inp.set1, inp.set2, dataset.id)
	out.set <- result$out.set
	step.num <- result$step.num

	if(is.interactive()) {
		# Execute the operation.
		result <- ExeAddFeatureSet(step.num)
		out.set <- result$out.set
		
	}	
	if(is.updateprogram()) {
		inp.set1.fs <- paste0("fs", inp.set1$feat.id)
		inp.set2.fs <- paste0("fs", inp.set2$feat.id)
		pgm.str <- paste0("fs", result$out.set$feat.id, " <- ", "AddFeatureSet(inp.set1 = ", inp.set1.fs, ", inp.set2 = ", inp.set2.fs, ", dataset.id = ", dataset.id, ")")
		AddProgramLine(pgm.str)

	}
	return(out.set)

}



DelFeatureSet <- function(inp.set1, inp.set2, dataset.id) {
	
	# Error handling.
	if(length(inp.set1) == 0) {
		stop("Argument inp.set1 cannot be of length 0")
	}
	
	if(length(inp.set2) == 0) {
		stop("Argument inp.set2 cannot be of length 0")
	}
	
	result <- SkelDelFeatureSet(inp.set1, inp.set2, dataset.id)
	out.set <- result$out.set
	step.num <- result$step.num

	if(is.interactive()) {
		# Execute the operation.
		result <- ExeDelFeatureSet(step.num)
		out.set <- result$out.set
		
	}	

	if(is.updateprogram()) {
		inp.set1.fs <- paste0("fs", inp.set1$feat.id)
		inp.set2.fs <- paste0("fs", inp.set2$feat.id)
		pgm.str <- paste0("fs", result$out.set$feat.id, " <- ", "DelFeatureSet(inp.set1 = ", inp.set1.fs, ", inp.set2 = ", inp.set2.fs, ", dataset.id = ", dataset.id, ")")
		AddProgramLine(pgm.str)

	}
	return(out.set)

}


IntersectFeatureSet <- function(inp.set1, inp.set2, dataset.id) {
	
	# Error handling.
	if(length(inp.set1) == 0) {
		stop("Argument inp.set1 cannot be of length 0")
	}
	
	if(length(inp.set2) == 0) {
		stop("Argument inp.set2 cannot be of length 0")
	}
	
	result <- SkelIntersectFeatureSet(inp.set1, inp.set2, dataset.id)
	out.set <- result$out.set
	step.num <- result$step.num

	if(is.interactive()) {
		# Execute the operation.
		result <- ExeIntersectFeatureSet(step.num)
		out.set <- result$out.set
		
	}	
	if(is.updateprogram()) {
		inp.set1.fs <- paste0("fs", inp.set1$feat.id)
		inp.set2.fs <- paste0("fs", inp.set2$feat.id)
		pgm.str <- paste0("fs", result$out.set$feat.id, " <- ", "IntersectFeatureSet(inp.set1 = ", inp.set1.fs, ", inp.set2 = ", inp.set2.fs, ", dataset.id = ", dataset.id, ")")
		AddProgramLine(pgm.str)

	}
	return(out.set)

}

BestK <- function(inp.list, k.val, dataset.id) {
	
	# Error handling
	if(is.null(inp.list) == TRUE) {
		stop('Argument inp.list cannot be NULL')
	}


	result <- SkelBestK(inp.list, k.val, dataset.id)
	out.set <- result$out.set
	step.num <- result$step.num

	if(is.interactive() == TRUE) {
		# Execute the operation
		result <- ExeBestK(step.num)
		out.set <- result$out.set
	}
	if(is.updateprogram()) {
		inp.list.fm <- paste0("fm",inp.list$list.id)
		pgm.str <- paste0("fs", result$out.set$feat.id, " <- ", "BestK(inp.list = ", inp.list.fm, ", k.val = ", k.val, ", dataset.id = ", dataset.id, ")")
		AddProgramLine(pgm.str)

	}
	return(out.set)


}

CreateFeature <- function(inp.feat1, inp.feat2, dataset.id) {
	
	# Error handling
	if(length(inp.feat1) != 1) {
		stop('Argument inp.feat1 is of length > 1')
	}
	
	if(length(inp.feat2) != 1) {
		stop('Argument inp.feat2 is of length > 1')
	}

	feat.id1 <- inp.feat1
	feat.id2 <- inp.feat2
	
	if(is.character(inp.feat1) == TRUE) {
		feat.id1 <- GetFeatureIds(inp.feat1)
	}
	if(is.character(inp.feat2) == TRUE) {
		feat.id2 <- GetFeatureIds(inp.feat2)
	}
	res <- ExeCreateFeature(feat.id1, feat.id2, dataset.id)
	out.set <- res$out.set
	if(is.updateprogram()) {
		inp.set1.fs <- paste0("fs", inp.set1$feat.id)
		inp.set2.fs <- paste0("fs", inp.set2$feat.id)
		pgm.str <- paste0("fs", result$out.set$feat.id, " <- ", "CreateFeature(inp.set1 = ", inp.set1.fs, ", inp.set2 = ", inp.set2.fs, ", dataset.id = ", dataset.id, ")")
		AddProgramLine(pgm.str)

	}
	return(out.set)
	
}
