CreateBatches <- function(node.list) {
	batch.list <- list()
	for(node in node.list) {
		if( (is.null(node) == FALSE) && (node$completed == 0) && (node$data.aware == 1)) {
			step.num <- node$step.num
			if(are.feat.inputs.computed(step.num) == TRUE) {
				level <- node$level
				
				if(level == LEVEL_PENDING) {
					stop('Level is set to level pending')
				} else {
					batch.list[level] <- list(union(unlist(batch.list[level]), step.num))
				}
			}
		}
	}
	return(batch.list)
}


GetFeatureSetForBatch <- function(batch) {
	res.set <- c()
	batch <- unlist(batch)
	for(step.num in batch) {
		tmp.set <- c()

		feat.id <- GetInputSetId.1(step.num)
		if(feat.id != MNULL) {
			set.val <- GetFeatureSet(feat.id)
			tmp.set <- union(tmp.set, unlist(set.val))
		}	
		
		feat.id <- GetInputSetId.2(step.num)
		if(feat.id != MNULL) {
			set.val <- GetFeatureSet(feat.id)
			if(is.stepadd.op(step.num)) {
				
				# If it is a step add operation, then the feature set to be
				# used will depend on the mask set.
			
				mask.set.val <- unlist(set.val)
				dataset.id <- GetDatasetIdFromStepNum(step.num)
				glb.feat.indices <- GetFeatureIndices(dataset.id)
				valid.set <- setdiff(glb.feat.indices, mask.set.val)
				tmp.set <- union(tmp.set, unlist(valid.set))
	
				
			} else {
				tmp.set <- union(tmp.set, unlist(set.val))
			}
		}
		
		res.set <- union(res.set, tmp.set)
	}
	return(res.set)
	
}

GetFeatureSetForBatchList <- function(batch.list) {
	feat.set.list <- list()
	len.batches <- length(batch.list)
	
	for(batch in 1:len.batches) {
		feat.set <- GetFeatureSetForBatch(batch.list[[batch]])
		feat.set.list[batch] <- list(feat.set)
	}
	return(feat.set.list)
}

GetRepetitionList <- function(feat.set.list) {
	
	rep.list <- list()
	for(feat.set in feat.set.list) {
		key <- NumVec2Str(feat.set)
		if(is.null(unlist(rep.list[[key]]))) {
			value <- 1
		} else {
			value <- value + 1
		}
		rep.list[[key]] <- value
	}
	return(rep.list)
}

DefaultBatchTablesMap <- function(len.batch.list, dataset.id) {
	batch.tables.map <- list()
	for(idx in 1:len.batch.list) {
		batch.tables.map[idx] <- list(GetFeatureIndices(dataset.id))
	}
	return(batch.tables.map)
}

PreFillMaterializationInfo <- function(cost.table, dir.table) {
	
	cost.table[1, 1] <- 0
	dir.table[1, 1] <- NumVec2Str(c(-1, -1))
	
	return(list(cost.table = cost.table, dir.table = dir.table))
}


ReadCost <- function(feat.set, dataset.id) {
	read.cost.all.feats <- GetReadCostForAllFeatures(dataset.id)
	read.intercept <- GetReadIntercept(dataset.id)
	
	len.feat.set <- length(unlist(feat.set))
	len.glb.set <- length(GetFeatureIndices(dataset.id))
	
	cost <- read.intercept + (len.feat.set/len.glb.set) * read.cost.all.feats
	return(cost)
}

WriteCost <- function(feat.set, dataset.id) {
	write.cost.all.feats <- GetWriteCostForAllFeatures(dataset.id)
	write.intercept <- GetWriteIntercept(dataset.id)
	
	len.feat.set <- length(unlist(feat.set))
	len.glb.set <- length(GetFeatureIndices(dataset.id))
	
	cost <- write.intercept + (len.feat.set/len.glb.set)* write.cost.all.feats
	return(cost)
	
}

StoreCost <- function(store.f, using.f, dataset.id) {
	cost.1 <- ReadCost(using.f, dataset.id)
	cost.2 <- WriteCost(store.f, dataset.id)
	return(cost.1 + cost.2)	

}

GetMinMatCost <- function(cost.vec, rep.list, in.feat.set, row.num, dataset.id) {
	glb.feat.indices <- GetFeatureIndices(dataset.id)
	set.to.use <- NA

	if(row.num == 1 && cost.vec[1] == 0) {
		set.to.use <- glb.feat.indices
		col.num <- row.num
		min.cost <- StoreCost(in.feat.set, glb.feat.indices, dataset.id)
		
		num.repetitions <- rep.list[[NumVec2Str(in.feat.set)]] + cost.vec[1]
		#min.cost <- min.cost + (num.repetitions * ReadCost(in.feat.set, dataset.id))
		str.set <- NumVec2Str(c(row.num, col.num))
		
	} else {
		min.cost <- Inf
		col.num <- -1
		
		for(j in 1:row.num) {
			if(j == 1) {
				# special case: use global feature set
				using.f <- glb.feat.indices
			} else {
				using.f <- Str2NumVec(names(rep.list[j - 1]))
			}
			cost <- StoreCost(in.feat.set, using.f, dataset.id) + cost.vec[j]

			if(min.cost > cost) {
				min.cost <- cost
				col.num <- j
				set.to.use <- using.f
				str.set <- NumVec2Str(c(row.num, col.num))
			}
		}
	}
	
	return(list(str.set = str.set, cost = min.cost, using.f = set.to.use))
}

GetMatInterimTables <- function(rep.list, dataset.id) {
	num.feature.sets <- length(rep.list)

	cost.table <- matrix(nrow = num.feature.sets + 1, ncol = num.feature.sets + 1)	
	dir.table <- matrix(nrow = num.feature.sets + 1, ncol = num.feature.sets + 1)	
	mat.from <- matrix(nrow = num.feature.sets + 1, ncol = num.feature.sets + 1)

	prefill.out <- PreFillMaterializationInfo(cost.table, dir.table)
	cost.table <- prefill.out$cost.table
	dir.table <- prefill.out$dir.table

	
	for(i in 2:nrow(cost.table)) {
		for(j in 1:i) {
			if(i == j) { # diagonal case
				if(is.na(cost.table[i, j])) {
					feat.set <- Str2NumVec(names(rep.list)[i - 1])	
					cost.vec <- cost.table[i - 1, c(1:(i - 1))]
					res <- GetMinMatCost(cost.vec, rep.list, feat.set, i - 1, dataset.id) 
					mat.from[i, j] <- NumVec2Str(res$using.f)
					
					cost <- res$cost
					num.repetitions <- rep.list[[NumVec2Str(feat.set)]]
					cost <- cost + ReadCost(feat.set, dataset.id) * num.repetitions

					cost.table[i, j] <- cost
					dir.table[i, j] <- res$str.set
				}
			} else { # non-diagonal case
				if(is.na(cost.table[i, j])) {
					if(j == 1) {
						mat.feat.set <- GetFeatureIndices(dataset.id)
						
					} else {
						mat.feat.set <- Str2NumVec(names(rep.list)[i - 2])
					}	
					feat.set <- Str2NumVec(names(rep.list)[i - 1])	
					num.repetitions <- rep.list[[NumVec2Str(feat.set)]]
					cost.table[i, j] <- cost.table[i - 1, j] + ReadCost(mat.feat.set, dataset.id) * num.repetitions
					dir.table[i, j] <- NumVec2Str(c(i - 1, j))
				}	
			}
		}
	}
	return(list(cost.table = cost.table, dir.table = dir.table, mat.from = mat.from))
}

RevAssignment <- function(assignment) {
	rev.assg <- list()
	len.assg <- length(assignment)
	idx <- len.assg
	for(i in c(1:len.assg)) {
		rev.assg[idx] <- assignment[i]
		idx <- idx - 1
	}
	return(rev.assg)
}
IndexRepetitions <- function(feat.set.list) {
	index.rep.list <- list()
	index <- 0
	for(feat.set in feat.set.list) {
		index <- index + 1
		key <- NumVec2Str(feat.set)
		if(is.null(unlist(index.rep.list[[key]]))) {
			value <- index
		} else {
			value <- index.rep.list[[key]]
			value <- union(value, index)
		}
		index.rep.list[[key]] <- value
	}
	return(index.rep.list)
}
GetBatchTablesMap <- function(rep.list, cost.table, dir.table, feat.set.list, dataset.id) {
	
	nr <- nrow(cost.table)
	min.idx <- rank(cost.table[nr, ])[1]
	
	mat.descisions <- list()
	mat.using.f <- list()

	
	max.hops <- nr - 1
	cnt <- 1

	row <- nr
	col <- min.idx
	
	feat.set.for.batch <- list()
	
	index <- 1
	
	while(cnt <= max.hops) {
		dir.vec <- Str2NumVec(dir.table[row, col])
		row <- dir.vec[1]
		col <- dir.vec[2]
		
		idx <- col - 1
		if(idx > 0) {
			feat.set <- Str2NumVec(names(rep.list)[idx])
			feat.set.for.batch[index] <- list(feat.set)
		} else {
			feat.set <- GetFeatureIndices(dataset.id)
			feat.set.for.batch[index] <- list(feat.set)
		}
		index <- index + 1
		cnt <- cnt + 1
	}
	
	# Rev the assignment
	feat.set.for.batch <- RevAssignment(feat.set.for.batch)
	index.rep.list <- IndexRepetitions(feat.set.list)

	
	# Expand the assignment
	temp.list <- list()
	for(i in 1:length(index.rep.list)) {
		for(j in index.rep.list[[i]]) {
			temp.list[j] <- feat.set.for.batch[i]
		}
	}	
	
	feat.set.for.batch <- temp.list
	return(feat.set.for.batch)
}

GetMaterializationDecisions <- function(rep.list, feat.set.list, dataset.id) {


	int.tables <- GetMatInterimTables(rep.list, dataset.id)
	
	dir.table <- int.tables$dir.table
	cost.table <- int.tables$cost.table
	mat.from <- int.tables$mat.from

	batch.tables.map <- GetBatchTablesMap(rep.list, cost.table, dir.table, feat.set.list, dataset.id)
	return(batch.tables.map)				
	
}

