#Copyright 2013 Pradap  and Arun Kumar and Christopher Re
#
#Licensed under the Apache License, Version 2.0 (the "License");
#you may not use this file except in compliance with the License.
#You may obtain a copy of the License at
#
#http://www.apache.org/licenses/LICENSE-2.0
#
#Unless required by applicable law or agreed to in writing, software
#distributed under the License is distributed on an "AS IS" BASIS,
#WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#See the License for the specific language governing permissions and
#limitations under the License.
#
#
CheckIfIgdConfigPresent <- function(step.num) {
	prog.ctr <- GetProgramCtr(step.num)
	program.id <- GetProgramId()
	
	res <- bSelect("ID", "IGD_CONFIG", "PROGRAM_CTR", prog.ctr, "PROGRAM_ID", program.id)
	if(length(res$ID) == 0)
		return(NULL)
	else
		return(res$ID)
}

CheckIfCgConfigPresent <- function(step.num) {
	prog.ctr <- GetProgramCtr(step.num)
	program.id <- GetProgramId()
	
	res <- bSelect("ID", "CG_CONFIG", "PROGRAM_CTR", prog.ctr, "PROGRAM_ID", program.id)
	if(length(res$ID) == 0)
		return(NULL)
	else
		return(res$ID)
}

StoreIgdConfig <- function(step.num) {

	res <- CheckIfIgdConfigPresent(step.num)
	if(is.null(res) == FALSE)
		return(res)
		

	# table name
	table.name <- "IGD_CONFIG"
	
	# fill the attribute names
	column.names <- FillAttrnames(table.name)
	#attr.names <- GetAttrnames(table.name)
	#len.attr.names <- length(attr.names)
	#column.names <- as.list(attr.names[2:len.attr.names]) # leave the id
		
	# Get the results
	dataset.id <- GetDatasetIdFromStepNum(step.num)
	igd.config <- GetIgdConfig(step.num)
	prog.ctr <- igd.config$program.ctr
	num.iters <- igd.config$num.iters
	step.size <- igd.config$step.size
	decay <- igd.config$decay
	init.wt <- igd.config$init.wt
	num.folds <- igd.config$num.folds

	# fill the results
	val <- list()
	val[[1]] <- dataset.id
	val[[2]] <- step.num
	val[[3]] <- prog.ctr
	val[[4]] <- num.iters
	val[[5]] <- step.size
	val[[6]] <- decay
	val[[7]] <- init.wt
	val[[8]] <- num.folds
	val[[9]] <- GetProgramId()
	
	bInsert(table.name, column.names, val)
	res <- bCount(table.name)
	return(res)
	
}

StoreCgConfig <- function(step.num) {
	
	res <- CheckIfCgConfigPresent(step.num)
	if(is.null(res) == FALSE)
		return(res)
	
	# table.name
	table.name <- "CG_CONFIG"
	# fill the attribute names
	column.names <- FillAttrnames(table.name)
	#Get the results
	program.id <- GetProgramId()
	dataset.id <- GetDatasetIdFromStepNum(step.num)
	cg.config <- GetCgConfig(step.num)
	prog.ctr <- cg.config$program.ctr
	num.iters <- cg.config$num.iters
	init.wt <- cg.config$init.wt
	num.folds <- cg.config$num.folds

	# fill the results
	val <- list()
	val[[1]] <- program.id
	val[[2]] <- dataset.id
	val[[3]] <- step.num
	val[[4]] <- prog.ctr
	val[[5]] <- num.iters
	val[[6]] <- init.wt
	val[[7]] <- num.folds

	bInsert(table.name, column.names, val)
	res <- bMaxId(table.name)
	return(res)
	
}

StoreIgdCoeff <- function(step.num, inp.feat.indices = NULL) {
	
	library(digest)
	
	table.name <- "IGD_COEFF"
	
	# fill the attr.names
	column.names <- FillAttrnames(table.name)
	
	# Update the igd config 
	igd.config.id <- StoreIgdConfig(step.num)	
	

	feat.id <- GetInputSetId.1(step.num)
	set.val <- unlist(GetFeatureSet(feat.id))
	set.val.str <- paste(set.val, collapse=",")
	res <- SetUpForIgdCoeffLearner(step.num)

	list.id <- GetOutputListId(step.num)
	values <- unlist(GetFeatureMapValues(list.id)[[1]])
	value.str <- paste0(values, collapse=",")
	# Get the results
	program.id <- GetProgramId()
	session.id <- GetSessionId()
	program.ctr <- GetProgramCtr(step.num)
	dataset.id <- GetDatasetIdFromStepNum(step.num)
	config.id <- igd.config.id

	if(is.null(inp.feat.indices))
		feat_indices <- SingleQuoteWrap(set.val.str)
	else
		feat_indices <- SingleQuoteWrap(inp.feat.indices)

	#feat_str <- SingleQuoteWrap(res$init.string)
	feat_str <- SingleQuoteWrap(strtrim(res$init.string, min(3000, nchar(res$init.string))))
	str_hash <- SingleQuoteWrap(digest(res$init.string))
	#value <- SingleQuoteWrap(value.str)
	value <- SingleQuoteWrap(strtrim(value.str, min(3000, nchar(value.str))))
	
	# fill the results
	val <- list()
	val[[1]] <- program.id
	val[[2]] <- session.id
	val[[3]] <- step.num
	val[[4]] <- program.ctr	
	val[[5]] <- dataset.id
	val[[6]] <- config.id
	val[[7]] <- feat_indices
	val[[8]] <- feat_str
	val[[9]] <- str_hash
	val[[10]] <- value
	
	bInsert(table.name, column.names, val)
	
	res <- bMaxId(table.name)
	return(res)
}

StoreCgCoeff <- function(step.num, inp.feat.indices = NULL) {
	library(digest)
	table.name <- "CG_COEFF"
	
	# fill the attr.names
	column.names <- FillAttrnames(table.name)
	
	# update cg_config
	cg.config.id <- StoreCgConfig(step.num)

	feat.id <- GetInputSetId.1(step.num)
	set.val <- unlist(GetFeatureSet(feat.id))
	set.val.str <- paste(set.val, collapse=",")
	res <- SetUpForCgCoeffLearner(step.num)
	
	list.id <- GetOutputListId(step.num)
	values <- unlist(GetFeatureMapValues(list.id)[[1]])
	value.str <- paste0(values, collapse=",")
	
	# Get the results
	program.id <- GetProgramId()
	session.id <- GetSessionId()
	prog.ctr <- GetProgramCtr(step.num)
	dataset.id <- GetDatasetIdFromStepNum(step.num)
	config.id <- cg.config.id

	if(is.null(inp.feat.indices))
		feat_indices <- SingleQuoteWrap(set.val.str)
	else
		feat_indices <- SingleQuoteWrap(inp.feat.indices)

	feat_str <- SingleQuoteWrap(strtrim(res$init.string, min(3000, nchar(res$init.string))))
	str_hash <- SingleQuoteWrap(digest(res$init.string))
	value <- SingleQuoteWrap(strtrim(value.str, min(3000, nchar(value.str))))

	
	# fill the results
	val <- list()
	val[[1]] <- program.id
	val[[2]] <- session.id
	val[[3]] <- step.num
	val[[4]] <- prog.ctr
	val[[5]] <- dataset.id
	val[[6]] <- config.id
	val[[7]] <- feat_indices
	val[[8]] <- feat_str
	val[[9]] <- str_hash
	val[[10]] <- value
	
	bInsert(table.name, column.names, val)
	res <- bCount(table.name)
	return(res)
} 
