
#Copyright 2013 Pradap  and Arun Kumar and Christopher Re
#
#Licensed under the Apache License, Version 2.0 (the "License");
#you may not use this file except in compliance with the License.
#You may obtain a copy of the License at
#
#http://www.apache.org/licenses/LICENSE-2.0
#
#Unless required by applicable law or agreed to in writing, software
#distributed under the License is distributed on an "AS IS" BASIS,
#WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#See the License for the specific language governing permissions and
#limitations under the License.
#
#
# File contains the data layer code for provenance.

StoreCorrelationResult <- function(step.num) {
	
	library(digest)
	dataset.id <- GetDatasetIdFromStepNum(step.num)
	list.id <- GetOutputListId(step.num)

	feature.list <- GetFeatureMapFeatureSet(list.id)[[1]]
	values <- unlist(GetFeatureMapValues(list.id)[[1]])
	table.name <- "CORRELATION"
	column.names <- list()

	
	column.names[[1]] <- "dataset_id"
	column.names[[2]] <- "feat_id1"
	column.names[[3]] <- "feat_id2"
	column.names[[4]] <- "feat_str"
	column.names[[5]] <- "str_hash"
	column.names[[6]] <- "value"
	
	len <- length(feature.list)
	
	for(i in 1:len) {
		feat.indices <- sort(feature.list[[i]])
		feat.str <- paste(feat.indices, collapse=",")
		str_hash <- digest(feat.str)
		corr_val <- GetValidValue(values[i])

		val <- list()
		# fill the results
		val[[1]] <- dataset.id
		val[[2]] <- feat.indices[1]
		val[[3]] <- feat.indices[2]
		val[[4]] <- SingleQuoteWrap(feat.str)
		val[[5]] <- SingleQuoteWrap(str_hash)
		val[[6]] <- corr_val
		res <- bInsert(table.name, column.names, val)
	}
	return(res)
}


StoreCorrelationXResult <- function(step.num) {
	library(digest)
	
	dataset.id <- GetDatasetIdFromStepNum(step.num)
	feat.id <- GetInputSetId.1(step.num)
	set.val <- unlist(GetFeatureSet(feat.id))
	set.val.str <- paste(set.val, collapse=",")
	res <- SetUpForCorrelationX(step.num)

	# Fix the table name
	table.name <- "CORRELATIONX"

	
	# fill the attribute names
	attr.names <- GetAttrnames(table.name)
	len.attr.names <- length(attr.names)
	column.names <- as.list(attr.names[2:len.attr.names]) # leave the id

	
	# Get the Results
	program.id <- GetProgramId()
	session.id <- GetSessionId()
	list.id <- GetOutputListId(step.num)
	values <- unlist(GetFeatureMapValues(list.id)[[1]])
	corrx_vals <- paste(values, collapse=",")

	# fill the results.
	val <- list()
	val[[1]] <- program.id
	val[[2]] <- session.id
	val[[3]] <- step.num
	val[[4]] <- dataset.id
	val[[5]] <- SingleQuoteWrap(set.val.str) # feat.indices
	val[[6]] <- SingleQuoteWrap(res$init.string) # feat.str
	val[[7]] <- SingleQuoteWrap(digest(res$init.string)) # str_hash
	val[[8]] <- SingleQuoteWrap(corrx_vals) # values

	bInsert(table.name, column.names, val)

}

StoreCorrelationYResult <- function(step.num) {
	library(digest)
	
	dataset.id <- GetDatasetIdFromStepNum(step.num)
	feat.id1 <- GetInputSetId.1(step.num)
	feat.id2 <- GetInputSetId.2(step.num)
	set.val1 <- unlist(GetFeatureSet(feat.id1))
	set.val1.str <- paste(set.val1, collapse=",")

	set.val2 <- unlist(GetFeatureSet(feat.id2))
	set.val2.str <- paste(set.val2, collapse=",")
	res <- SetUpForCorrelationY(step.num)


	# table.name 
	table.name <- "CORRELATIONY"
	
	# attribute names
	attr.names <- GetAttrnames(table.name)
	len.attr.names <- length(attr.names)
	column.names <- as.list(attr.names[2:len.attr.names]) # leave the id.

	# results
	program.id <- GetProgramId()
	session.id <- GetSessionId()
	list.id <- GetOutputListId(step.num)
	values <- unlist(GetFeatureMapValues(list.id)[[1]])
	corry_vals <- paste(values, collapse=",")
	

	# fill the results.
	val <- list()
	val[[1]] <- program.id
	val[[2]] <- session.id
	val[[3]] <- step.num
	val[[4]] <- dataset.id
	val[[5]] <- SingleQuoteWrap(set.val1.str)
	val[[6]] <- SingleQuoteWrap(set.val2.str)
	val[[7]] <- SingleQuoteWrap(res$init.string)
	val[[8]] <- SingleQuoteWrap(digest(res$init.string))
	val[[9]] <- SingleQuoteWrap(corry_vals)
	
	bInsert(table.name, column.names, val)
		
}
