
#Copyright 2013 Pradap  and Arun Kumar and Christopher Re
#
#Licensed under the Apache License, Version 2.0 (the "License");
#you may not use this file except in compliance with the License.
#You may obtain a copy of the License at
#
#http://www.apache.org/licenses/LICENSE-2.0
#
#Unless required by applicable law or agreed to in writing, software
#distributed under the License is distributed on an "AS IS" BASIS,
#WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#See the License for the specific language governing permissions and
#limitations under the License.
#
#
# Function to find pair wise correlation among the features given in the input set
#
# Args:
#  inp.set : Feature set containing the feature ids from a dataset.
#  dataset.id : Id of the dataset to which the features belong to.
# 
# Returns:
#  List from the feature map.
#  TODO(pradap): Return a type specific to correlation s.t, printing the result becomes simpler.


CorrelationX <- function(inp.set, dataset.id) {
	
	# Error handling.
	if(is.null(inp.set)) {
		stop('Argument inp.set cannot be NULL')
	}
	res <- SkelCorrelationX(inp.set, dataset.id)
	
	res.list <- res$out.list
	step.num <- res$step.num
	
	if(is.interactive() == TRUE) {
		RunInteractiveOperation(step.num, dataset.id)
	}

	ret.list <- GetFeatureMap(res.list$list.id)
	if(is.updateprogram()) {
		inp.set.fs <- paste0("fs", inp.set$feat.id)
		pgm.str <- paste0("fm", res.list$list.id, " <- ", "CorrelationX(inp.set = ", inp.set.fs, ", dataset.id = ", dataset.id, ")")
		AddProgramLine(pgm.str)

	}
	
	return(ret.list)
}

CorrelationY <- function(inp.set1, inp.set2, dataset.id) {
	
	# Error handling.
	if(is.null(inp.set1) || is.null(inp.set2)) {
		stop('Argument inp.set cannot be NULL')
	}
	res <- SkelCorrelationY(inp.set1, inp.set2, dataset.id)
	
	res.list <- res$out.list
	step.num <- res$step.num
	
	if(is.interactive() == TRUE) {
		RunInteractiveOperation(step.num, dataset.id)
	}

	ret.list <- GetFeatureMap(res.list$list.id)
	if(is.updateprogram()) {
		inp.set1.fs <- paste0("fs", inp.set1$feat.id)
		inp.set2.fs <- paste0("fs", inp.set2$feat.id)
		pgm.str <- paste0("fm", res.list$list.id, " <- ", "CorrelationY(inp.set1 = ", inp.set1.fs, ", inp.set2 = ", inp.set2.fs ,", dataset.id = ", dataset.id, ")")
		AddProgramLine(pgm.str)

	}
	
	return(ret.list)
}


CoeffLearner <- function(inp.set, type, dataset.id, num.iters = 2, step.size = 0.01, decay = 1,
init.wt = 0, num.folds = 1) {
	
	# Error handling.
	if(is.null(inp.set)) {
		stop('Argument inp.set cannot be NULL')
	}
	
	if(num.folds != 1) {
		stop('Argument num.folds must be set to 1')
	}
	type <- tolower(type)
	if(type == "igd") {
		res <- SkelIgdCoeffLearner(inp.set, num.iters, step.size, decay, init.wt, num.folds,
dataset.id)
	} else if(type == "cg") {
		res <- SkelCgCoeffLearner(inp.set, num.iters, init.wt, num.folds, dataset.id)
	}

	res.list <- res$out.list
	step.num.list <- res$step.num.list

	if(is.interactive() == TRUE) {
		RunInteractiveOperation(step.num.list, dataset.id)
	}
	

	if(is.updateprogram()) {
		inp.set.fs <- paste0("fs", inp.set$feat.id)
		pgm.str <- paste0("fm", res.list$list.id, " <- ", "CoeffLearner(inp.set = ", inp.set.fs, ", type = \"", type ,"\", dataset.id = ", dataset.id, ", num.iters = ", num.iters, ", step.size = ", step.size, ", decay = ", decay ,")")
		AddProgramLine(pgm.str)

	}
	#ret.list <- GetFeatureMap(res.list$list.id)
	return(res.list)

}

Evaluate <- function(inp.set, train.type, eval.type, dataset.id, num.iters = 2, step.size = 0.01, decay = 1, init.wt
= 0, num.folds = 1) {
	
	# Error handling
	if(is.null(inp.set)) {
		stop('Argument inp.set cannot be NULL')
	}
	
	type <- tolower(train.type)
	
	if(type == "igd") {
		res <- SkelIgdEvaluate(inp.set, eval.type, num.iters, step.size, decay, init.wt, num.folds, dataset.id)
	} else if(type == "cg") {
		res <- SkelCgEvaluate(inp.set, eval.type, num.iters, init.wt, num.folds, dataset.id)
	}

	res.list <- res$out.list
	step.num.list <- res$step.num.list

	if(is.interactive() == TRUE) {
		RunInteractiveOperation(step.num.list, dataset.id)
	}
	if(is.updateprogram()) {
		inp.set.fs <- paste0("fs", inp.set$feat.id)
		pgm.str <- paste0("fm", res.list$list.id, " <- ", "Evaluate(inp.set = ", inp.set.fs, ", train.type = \"", train.type , "\", eval.type = \"", eval.type ,"\", dataset.id = ", dataset.id, ", num.iters = ", num.iters, ", step.size = ", step.size,", init.wt = ", init.wt ,", decay = ", decay ,")")
		AddProgramLine(pgm.str)

	}
	
	#ret.list <- GetFeatureMap(res.list$list.id)
	return(res.list)
}


