
#Copyright 2013 Pradap  and Arun Kumar and Christopher Re
#
#Licensed under the Apache License, Version 2.0 (the "License");
#you may not use this file except in compliance with the License.
#You may obtain a copy of the License at
#
#http://www.apache.org/licenses/LICENSE-2.0
#
#Unless required by applicable law or agreed to in writing, software
#distributed under the License is distributed on an "AS IS" BASIS,
#WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#See the License for the specific language governing permissions and
#limitations under the License.
#
#
SetUpForIgdCoeffLearner <- function(step.num) {
	
	feat.id <- GetInputSetId.1(step.num)
	set.val <- unlist(GetFeatureSet(feat.id))

	igd.config <- GetIgdConfig(step.num)
	
	decay <- igd.config$decay
	step.size <- igd.config$step.size
	num.folds <- igd.config$num.folds

	inp.list.id <- GetInputList.1(step.num)

	# The number of feat.list is hardcoded to 1.
	
	if(inp.list.id == MNULL) {
		init.wt <- igd.config$init.wt
		values <- rep(init.wt, length(set.val))
		coeff.list <- list() # re-visit
		coeff.list[[1]] <- list()
		for(i in 1:num.folds) {
			coeff.list[[1]][[i]] <- values
		}
		
	} else {
		values <- GetFeatureMapValues(inp.list.id)
		values <- values[[1]]
		coeff.list <- values
	}

	param.vec <- c(step.size, decay, num.folds)
	
	res <- InitIgdCoeffLearner(param.vec, list(set.val), coeff.list)
	return(res)
	
}

SetUpForCgCoeffLearner <- function(step.num) {

	feat.id <- GetInputSetId.1(step.num)
	set.val <- GetFeatureSet(feat.id)
	feat.list <- set.val

	cg.config <- GetCgConfig(step.num)
	
	num.folds <- cg.config$num.folds
	
	inp.list.id <- GetInputList.1(step.num)
	
	# default values.
	def.u <- 0
	def.g <- 0
	
	if(inp.list.id == MNULL) {
		init.wt <- cg.config$init.wt
		len.set.val <- length(unlist(set.val))
		wt.val <- rep(init.wt, len.set.val)
		u.val <- rep(def.u, len.set.val)
		g.val <- rep(def.g, len.set.val)

		coeff.list <- list()
		g.list <- list()
		u.list <- list()
		coeff.list[[1]] <- list()
		u.list[[1]] <- list()
		g.list[[1]] <- list()
		for(i in 1:num.folds) {
			coeff.list[[1]][[i]] <- list()
			u.list[[1]][[i]] <- list()
			g.list[[1]][[i]] <- list()

			coeff.list[[1]][[i]] <- wt.val
			u.list[[1]][[i]] <- u.val
			g.list[[1]][[i]] <- g.val
		}
		
			
		
	} else {
		values <- GetFeatureMapValues(inp.list.id)
		values <- values[[1]]
		
		coeff.list <- values$coeff.list
		u.list <- values$u.list
		g.list <- values$g.list
	}

	param.vec <- c(num.folds)
	
	res <- InitCgCoeffLearner(param.vec, feat.list, coeff.list, u.list, g.list)
	return(res)
	
		
	
}

InitIgdCoeffLearner <- function(param.vec, feat.set.list, coeff.list) {
	# Supposed to be generic that can be used across evaluate and step functions.

	# format:
	# params#feat.set1.indices$fold1.coef.vals~fold2.coef.vals~..~foldn.coef.vals#feat.set2.indices$fold1.coef.vals
	
	param.vec.str <- paste(param.vec,  collapse = ",")
	accum.str <- param.vec.str
	
	decode.indices <- c()

	for(i in 1:length(feat.set.list)) {
		feat.set <- feat.set.list[[i]]
		decode.indices <- union(decode.indices, feat.set)
		accum.str <- paste(accum.str, paste(feat.set, collapse=","), sep = "#")
		accum.str <- paste(accum.str, "$", sep="")
		
		for(j in 1:length(coeff.list[[i]])) {
			accum.str <- paste(accum.str, paste(coeff.list[[i]][[j]], collapse=","), sep="")
			if(j != length(coeff.list[[i]]))
				accum.str <- paste(accum.str, "~", sep="")
		}
		
	}
	return(list(decode.indices = decode.indices, init.string = accum.str, param.vec = param.vec))
}


InitCgCoeffLearner <- function(param.vec, feat.set.list, coeff.list, u.list, g.list) {
	
	# format of the input:
	#  params#feat.set.indices1$fold1.coef.vals!fold1.u.vals!fold1.g.vals~fold2.coef.vals!fold2.u.vals!fold2.g.vals~..#feat.set.indices2!fold1.coef.vals..

	
	param.vec.str <- paste(param.vec, collapse=",")
	accum.str <- param.vec.str

	decode.indices <- c()
	
	for(i in 1:length(feat.set.list)) {
		
		feat.set <- feat.set.list[[i]]
		coeff <- coeff.list[[i]]
		u <- u.list[[i]]
		g <- g.list[[i]]

		decode.indices <- union(decode.indices, feat.set)
		accum.str <- paste(accum.str, paste(feat.set, collapse=","), sep="#")
		accum.str <- paste(accum.str, "$", sep="")

		for(j in 1:length(coeff)) {
			accum.str <- paste(accum.str, paste(coeff[[j]], collapse=","), sep="")
			accum.str <- paste(accum.str, "!", sep="")
			accum.str <- paste(accum.str, paste(u[[j]], collapse=","), sep="")
			accum.str <- paste(accum.str, "!", sep="")

			accum.str <- paste(accum.str, paste(g[[j]], collapse=","), sep="")
			if(j != length(coeff))
				accum.str <- paste(accum.str, "~", sep="")
			
		}
		
		
	}

	return(list(decode.indices = decode.indices, init.string = accum.str, feat.set.list =
feat.set.list, coeff.list = coeff.list, u.list = u.list, g.list = g.list, param.vec = param.vec))
}


ParseIgdCoeffLearner <- function(res.string, num.folds) {
	
	#format of res.string
	# feature.indices1$fold1.coeffval,..,foldn.coeffval#feature.indices2$fold1.coeffval,..,foldn.coeffval
	
	# output
	# 1) feat.set.list[[i]] contains feat.set.indices
	# 2) coeff.list[[[i]][1..num.folds]] will contain coeff.values for differnt folds.

	splitted.str <- strsplit(res.string, split="#")
	splitted.str <- splitted.str[[1]]
	num.feat.sets <- length(splitted.str)
	feat.set.list <- list()
	coeff.list <- list()
	
	for(i in 1:num.feat.sets) {
		feat.set.str <- strsplit(splitted.str[i], split="\\$")
		feat.set.str <- feat.set.str[[1]]
		
		feat.set.list[[i]] <- unlist(lapply(strsplit(feat.set.str[[1]], split=","), as.integer))
		coeff.res.str <- strsplit(feat.set.str[[2]], split="~")
		coeff.res.str <- coeff.res.str[[1]]

		if(length(coeff.res.str) != num.folds)
			stop('coeff.res length does not match with the num folds')
		
		coeff.list[[i]] <- list()
		for(j in 1:num.folds) {
			coeff.list[[i]][[j]] <- list()
			coeff.list[[i]][[j]] <- unlist(lapply(strsplit(coeff.res.str[[j]], split=","), as.numeric))
		}
	}
	return(list(feat.set.list = feat.set.list, coeff.list = coeff.list))
	
}


ParseCgCoeffLearner <- function(res.string, num.folds, feat.set.list, coeff.list, u.list, g.list) {
		
	# format of res.string
	# feature.indices1$fold1.denomWTerm!fold1.g.val~fold2.denomWTerm!fold2.g.val~..#feature.indices2$fold1.denoWTerm!fold1.g.val~fold2.denomWTerm!fold2.g.val..

	# output
	# 1) feat.set.list[[i]] contains feat.set.indices
	# 2) coeff.list[[i]][[1..num.folds]] contains co-efficient (weight) values
	# 3) u.list[[i]][[1..num.folds]] contains updated u vector
	# 4) g.list[[i]][[1..num.folds]] contains updated g vector


	
	splitted.str <- strsplit(res.string, split="#")
	splitted.str <- splitted.str[[1]]

	num.feat.sets <- length(splitted.str)

	feat.set.list <- list()
	denom.w.term <- list()
	upd.g.list <- list()
	upd.u.list <- list()
	upd.coeff.list <- list()

	
	for(i in 1:num.feat.sets) {
		feat.set.res.str <- strsplit(splitted.str[i], split="\\$")
		feat.set.res.str <- feat.set.res.str[[1]]
		set.idx = i

	
		feat.set.list[[i]] <- unlist(lapply(strsplit(feat.set.res.str[[1]], split=","), as.integer))

		coef.res.str <- strsplit(feat.set.res.str[[2]], split="~")
		coef.res.str <- coef.res.str[[1]]

		if(length(coef.res.str) != num.folds) {
			stop('coeff.res length does not match with the num.folds')
		}
		
		denom.w.term[[i]] <- list()
		upd.g.list[[i]] <- list()
		upd.u.list[[i]] <- list()
		upd.coeff.list[[i]] <- list()
		
		for(j in 1:num.folds) {
			fold.idx <- j
			denom.w.term[[i]][[j]] <- list()
			upd.g.list[[i]][[j]] <- list()
			upd.u.list[[i]][[j]] <- list()
			upd.coeff.list[[i]][[j]] <- list()

			tmp.str <- strsplit(coef.res.str[[j]], split = "!")
			tmp.str <- tmp.str[[1]]

			denom.w.term[[i]][[j]] <- as.numeric(unlist(tmp.str[[1]]))

			upd.g.list[[i]][[j]] <- unlist(lapply(strsplit(tmp.str[[2]], split=","), as.numeric))
			#upd.g.list[[i]][[j]] <- list(list(rnorm(length(unlist(upd.g.list[[i]][[j]])))))
			
			if(denom.w.term[[i]][[j]] == 0) {
				u.list[[i]][[j]] <- upd.g.list[[i]][[j]] 
				upd.u.list[[i]][[j]] <- upd.g.list[[i]][[j]] 
			}
			
			# beta = g'(g' - g)/u(g' - g)
			diff.g <- upd.g.list[[i]][[j]] - g.list[[i]][[j]]
			num <- upd.g.list[[i]][[j]] %*% diff.g
			den <- u.list[[i]][[j]] %*% diff.g
			beta <- num/den

			
			# update u vector
			# u' = g - beta*u
			if(denom.w.term[[i]][[j]] != 0)
				upd.u.list[[i]][[j]] <- upd.g.list[[i]][[j]] - beta * u.list[[i]][[j]]
		
			#upd.u.list[[i]][[j]] <- list(list(rnorm(length(unlist(upd.u.list[[i]][[j]])))))
			
			# update w vector
			# w' = w + ( g %*% u * u ) / denom.w.term
			num <- (upd.g.list[[i]][[j]] %*% u.list[[i]][[j]]) * u.list[[i]][[j]]
			
			if(denom.w.term[[i]][[j]] != 0) {
				upd.coeff.list[[i]][[j]] <- coeff.list[[i]][[j]] + (num/denom.w.term[[i]][[j]])
				#upd.coeff.list[[i]][[j]] <- rnorm(length(unlist(upd.coeff.list[[i]][[j]])))
			} else {
				upd.coeff.list[[i]][[j]] <- coeff.list[[i]][[j]] 
			}
			
		}

		
	}
	return(list(feat.set.list = feat.set.list, coeff.list = upd.coeff.list, u.list = upd.u.list, g.list = upd.g.list))
	
}




ResIgdCoeffLearner <- function(step.num, res.string) {
	exe.list <- SetUpForIgdCoeffLearner(step.num)
	num.folds <- exe.list$param.vec[3]

	list.id <- GetOutputListId(step.num)	
	res.list <- ParseIgdCoeffLearner(res.string, num.folds)
	SetFeatMapFeatSetList(list.id, res.list$feat.set.list)
	SetFeatureMapValues(list.id, res.list$coeff.list)
	
}

ResCgCoeffLearner <- function(step.num, res.string) {
	exe.list <- SetUpForCgCoeffLearner(step.num)
	num.folds <- exe.list$param.vec[1]

	list.id <- GetOutputListId(step.num)
	res.list <- ParseCgCoeffLearner(res.string, num.folds, exe.list$feat.set.list, exe.list$coeff.list, exe.list$u.list, exe.list$g.list)
	
	out.list <- list() # fit all the results in a single list.
	out.list$coeff.list <- res.list$coeff.list
	out.list$u.list <- res.list$u.list
	out.list$g.list <- res.list$g.list

	SetFeatMapFeatSetList(list.id, res.list$feat.set.list)
	SetFeatureMapValues(list.id, out.list)
}
