
#Copyright 2013 Pradap  and Arun Kumar and Christopher Re
#
#Licensed under the Apache License, Version 2.0 (the "License");
#you may not use this file except in compliance with the License.
#You may obtain a copy of the License at
#
#http://www.apache.org/licenses/LICENSE-2.0
#
#Unless required by applicable law or agreed to in writing, software
#distributed under the License is distributed on an "AS IS" BASIS,
#WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#See the License for the specific language governing permissions and
#limitations under the License.
#
#
SetUpForCorrelationX <- function(step.num) {
	feat.id <- GetInputSetId.1(step.num)
	set.val <- unlist(GetFeatureSet(feat.id))
	
	res <- InitCorrelationX(set.val)
	return(res)
	
}

SetUpForCorrelationY <- function(step.num) {
	feat.id1 <- GetInputSetId.1(step.num)
	feat.id2 <- GetInputSetId.2(step.num)
	
	set.val1 <- unlist(GetFeatureSet(feat.id1))
	set.val2 <- unlist(GetFeatureSet(feat.id2))

	res <- InitCorrelationY(set.val1, set.val2)
	return(res)
}

InitCorrelationX <- function(set.val) {

	featset.list <- combn(set.val, 2, simplify = FALSE)
	decode.indices <- set.val
	
	s1 <- paste(set.val, collapse=",")
	s2 <- paste(unlist(featset.list), collapse = ",")
	
	init.string <- paste(s1, s2, sep="#")
	
	return(list(featset.list = featset.list, decode.indices = decode.indices, init.string =
		init.string))	
				
}

InitCorrelationY <- function(set.val1, set.val2) {
	
	featset.list <- VectorJoin(set.val1, set.val2)
	decode.indices <- union(set.val1, set.val2)
	
	s1 <- paste(decode.indices, collapse=",")
	s2 <- paste(unlist(featset.list), collapse=",")
	
	init.string <- paste(s1, s2, sep = "#")
	
	return(list(featset.list = featset.list, decode.indices = decode.indices, init.string =
		init.string))
}

ResCorrelationX <- function(step.num, res.string) {
	
	exe.list <- SetUpForCorrelationX(step.num)
	res.list <- ParseCorrelationRes(res.string, exe.list$decode.indices, exe.list$featset.list)
	out.list.id <- GetOutputListId(step.num)
	SetFeatMapFeatSetList(out.list.id, exe.list$featset.list)
	SetFeatureMapValues(out.list.id, res.list)
	
}

ResCorrelationY <- function(step.num, res.string) {
	exe.list <- SetUpForCorrelationY(step.num)
	res.list <- ParseCorrelationRes(res.string, exe.list$decode.indices, exe.list$featset.list)
	out.list.id <- GetOutputListId(step.num)
	SetFeatMapFeatSetList(out.list.id, exe.list$featset.list)
	SetFeatureMapValues(out.list.id, res.list)
}

ParseCorrelationRes <- function(res.str, decode.indices, featset.list) {
	
	sum.x <- list()
	sum.sq.x <- list()
	prod.x <- list()
	corr.res <- list()

	splitted.str <- strsplit(res.str, split = "#")
	splitted.str <- splitted.str[[1]]
	
	num.tuples <- as.numeric(splitted.str[1])

	sum.x.str <- strsplit(splitted.str[2], split = ",")
	sum.x.str <- sum.x.str[[1]]
	
	sum.sq.x.str <- strsplit(splitted.str[3], split = ",")
	sum.sq.x.str <- sum.sq.x.str[[1]]

	prod.x.str <- strsplit(splitted.str[4], split = ",")
	prod.x.str <- prod.x.str[[1]]
	
	for(i in 1:length(decode.indices)) {
		idx <- decode.indices[i]
		sum.x[[idx]] <- as.numeric(sum.x.str[i])
		sum.sq.x[[idx]] <- as.numeric(sum.sq.x.str[i])
	}	
	for(i in 1:length(prod.x.str)) {
		prod.x[[i]] <- as.numeric(prod.x.str[i])
	}

	# Compute  Correlation
	for(i in 1:length(featset.list)) {
		a <- featset.list[[i]][1]
		b <- featset.list[[i]][2]
		
		num <- num.tuples * prod.x[[i]] - sum.x[[a]] * sum.x[[b]]
		den <- sqrt(num.tuples * sum.sq.x[[a]] - sum.x[[a]]^2) * sqrt(num.tuples * sum.sq.x[[b]] - sum.x[[b]]^2)
		corr.res[[i]] <- num/den
	}
	return(corr.res)
	
}

