
#Copyright 2013 Pradap  and Arun Kumar and Christopher Re
#
#Licensed under the Apache License, Version 2.0 (the "License");
#you may not use this file except in compliance with the License.
#You may obtain a copy of the License at
#
#http://www.apache.org/licenses/LICENSE-2.0
#
#Unless required by applicable law or agreed to in writing, software
#distributed under the License is distributed on an "AS IS" BASIS,
#WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#See the License for the specific language governing permissions and
#limitations under the License.
#
#
ExeAssignFeatureSet <- function(inp.feat.ids, dataset.id) {

	step.num <- GetStepNum() + 1
	
	# Insert into featureset db
	feat.id  <- InsertRowInTable('FeatureSetDB',
					data.table(step.num, list(inp.feat.ids)))  
	pgm.ctr <- step.num
	
	# Insert into program table
	#browser()
	inpset.id1 <- MNULL
	inpset.id2 <- MNULL
	inplist.id1 <- MNULL
		
	outset.id <- feat.id
	outlist.id <- MNULL
	
	
	res.step.num <- InsertRowInTable('Program', data.table(dataset.id, FEATURE_SET,
					step.num, pgm.ctr, inpset.id1, inpset.id2, inplist.id1, outset.id, outlist.id))
	
	# Create a feature set class and return the result.
	out.set <- FeatureSetClass(feat.id, list(inp.feat.ids))
	return(list(step.num = step.num, out.set = out.set))	
}

ExeAddFeatureSet <- function(step.num) {
	

	id1 <- GetInputSetId.1(step.num)
	id2 <- GetInputSetId.2(step.num)
	
	set.val1 <- unlist(GetFeatureSet(id1))
	set.val2 <- unlist(GetFeatureSet(id2))

	res.id <- GetOutputSetId(step.num)
	res.set <- union(set.val1, set.val2)

	SetFeatureSet(res.id, res.set)
	
	out.set <- FeatureSetClass(res.id, list(res.set))
	return(list(step.num = step.num, out.set = out.set))

}


ExeDelFeatureSet <- function(step.num) {
	

	id1 <- GetInputSetId.1(step.num)
	id2 <- GetInputSetId.2(step.num)
	
	set.val1 <- unlist(GetFeatureSet(id1))
	set.val2 <- unlist(GetFeatureSet(id2))

	res.id <- GetOutputSetId(step.num)
	res.set <- setdiff(set.val1, set.val2)

	SetFeatureSet(res.id, res.set)
	
	out.set <- FeatureSetClass(res.id, list(res.set))
	return(list(step.num = step.num, out.set = out.set))

}



ExeIntersectFeatureSet <- function(step.num) {
	

	id1 <- GetInputSetId.1(step.num)
	id2 <- GetInputSetId.2(step.num)
	
	set.val1 <- unlist(GetFeatureSet(id1))
	set.val2 <- unlist(GetFeatureSet(id2))

	res.id <- GetOutputSetId(step.num)
	res.set <- intersect(set.val1, set.val2)

	SetFeatureSet(res.id, res.set)
	
	out.set <- FeatureSetClass(res.id, list(res.set))
	return(list(step.num = step.num, out.set = out.set))

}

ExeBestK <- function(step.num) {

	list.id <- GetInputList.1(step.num)
	values <- GetFeatureMapValues(list.id)[[1]]

	if(is.element("coeff.list", names(values)) == TRUE) {
		coeff.list <- values$coeff.list
	} else {
		coeff.list <- values
	}	 
	
	coeff.list <- unlist(coeff.list)
	config <- GetBestKConfig(step.num)
	k.val <- config$k.val
	
	ordered.values <- order(coeff.list, decreasing = TRUE)
	top.k <- ordered.values[1:k.val]
	#browser()
	
	feat.set.values <- GetFeatureMapFeatureSet(list.id)[[1]]
	if(length(feat.set.values) != 1)
		stop('Length of feat.set.values is greater than 1')
	feat.set <- sort(unique(unlist(feat.set.values)[top.k]))
	
	res.id <- GetOutputSetId(step.num)
	
	SetFeatureSet(res.id, feat.set)
	out.set <- FeatureSetClass(res.id, list(feat.set))
	
	return(list(step.num = step.num, out.set = out.set))
	

		
}

ExeCreateFeature <- function(feat.id1, feat.id2, dataset.id) {
	
	step.num <- GetStepNum() + 1

	new.feature <- GetNewFeatureName(dataset.id)
	
	new.feature.name <- new.feature$feature.name
	new.feature.id   <- new.feature$feat.id
	
	AddFeatureToDataset(new.feature.name, new.feature.id, dataset.id)

	
	# Create and Insert in to feature set DB.
	inp.feat.id1 <- InsertRowInTable('FeatureSetDB', data.table(step.num, list(feat.id1)))
	inp.feat.id2 <- InsertRowInTable('FeatureSetDB', data.table(step.num, list(feat.id2)))
	out.feat.id <- InsertRowInTable('FeatureSetDB', data.table(step.num, list(new.feature.id)))
	
	pgm.ctr <- step.num
	inplist.id1 <- MNULL
	outlist.id <- MNULL

	res.step.num <- InsertRowInTable('Program', data.table(dataset.id, CREATE_FEATURE, step.num,
pgm.ctr, inp.feat.id1, inp.feat.id2, inplist.id1, out.feat.id, outlist.id))

	pgm.ctr <- step.num
	res.step.num <- InsertRowInTable('DerivedFeatures', data.table(dataset.id, step.num,
pgm.ctr, feat.id1, feat.id2, new.feature.id))

	out.set <- FeatureSetClass(out.feat.id, list(new.feature.id))
	return(list(step.num = step.num, out.set = out.set))
	
}
