
#Copyright 2013 Pradap  and Arun Kumar and Christopher Re
#
#Licensed under the Apache License, Version 2.0 (the "License");
#you may not use this file except in compliance with the License.
#You may obtain a copy of the License at
#
#http://www.apache.org/licenses/LICENSE-2.0
#
#Unless required by applicable law or agreed to in writing, software
#distributed under the License is distributed on an "AS IS" BASIS,
#WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#See the License for the specific language governing permissions and
#limitations under the License.
#
#
RunInteractiveOperation <- function(step.num.list, dataset.id) {
	
	node.list <- GenNodeList(dataset.id)
	idx <- 0
	for(step.num in step.num.list) {
		if(is.data.aware(step.num) == TRUE) {
			#stats
			idx <- idx + 1
			UpdateStatsList('batch.idx', idx) 	
			#stats
			
			res <- CheckAndReuseLocalResultForAStep(step.num, dataset.id)
		
			if(res == FALSE) {
				batch.tables.map <- GetFeatureIndices(dataset.id)
				#stats
				UpdateStatsOperatorClass(step.num)	
				#stats
				node.list <- RunDataAwareOperationForABatch(c(step.num), node.list, batch.tables.map, dataset.id)
			} else {
				#stats
				UpdateStatsList('mat.time', 0.0)	
				UpdateStatsList('init.time', 0.0)	
				UpdateStatsList('exe.time', 0.0)
				UpdateStatsList('retrieve.result', 0.0)
				#stats
				printf("Reused result for step num : %d", step.num)
			}


		} else { # Data agnostic operation.
			node.list <- RunDataAgnosticOperation(node.list, dataset.id)
		}
	}
}

RunDataAwareOperation <- function(batch.list, node.list, batch.tables.map, dataset.id) {
	for(idx in 1:length(batch.list)) {
		#stats
		UpdateStatsList('batch.idx', idx) 	
		#stats
		step.num.list <- batch.list[[idx]]

		#stats
		UpdateStatsOperatorClass(step.num.list)	
		#stats
		time.taken <- system.time(node.list <- RunDataAwareOperationForABatch(step.num.list, node.list,
batch.tables.map[[idx]], dataset.id), gcFirst = TRUE)
		#printf("RunDataAwareOpForaBatch took %f secs", time.taken[[3]])
	}
	return(node.list)
}

RunDataAwareOperationForABatch <- function(step.num.list, node.list, batch.tables.map, dataset.id) {
	
	tot.num.features <- GetTotalNumFeatures(dataset.id)
	orig.num.features <- GetOrigNumFeatures(dataset.id)
	target.idx <- GetTargetIndex(dataset.id)

	decode.indices <- c()
	obj.list <- ""
	
	#stats
	init.time <- 0
	#stats
	# DL calls start#
	bTerminate() 
	time.taken <- bInitFeatureLengths(tot.num.features, orig.num.features, target.idx)
	#stats
	#printf("InitFeatLengths took :%f secs", time.taken[[3]])
	init.time <- init.time + time.taken[[3]]
	#stats
	
	# DL calls end#

	obj.store <- list()
	
	for(step.num in step.num.list) {

		node <- node.list[[step.num]]
		
		if((node$data.aware == 1) && (node$completed == 0)) {
			
			if(are.inputs.computed(step.num) == TRUE) {
				
				#op.id <- GetOperatorId(step.num)
				obj <- GetExeObject(step.num)
				obj.store[[step.num]] <- obj
				res <- SetUpForOperation(step.num)
				
				# DL calls start#
				time.taken <- bInitInStages(paste(obj, res$init.string, sep = "@"))
				# DL calls end#
				# stats
				#printf("bInitInStages took :%f secs", time.taken[[3]])
				init.time <- init.time + time.taken[[3]]
				# stats
				
				decode.indices <- union(decode.indices, res$decode.indices)
				
				if(nchar(obj.list) == 0) 
					obj.list <- obj
				else
					obj.list <- paste(obj.list, obj, sep="@")
			
					
			}
		}
	}
		
	new.feat.str <- GetNewFeatString(dataset.id)
	
	str.decode.indices <- paste(sort(decode.indices), collapse = ",")
	# DL calls start#
	time.taken <- bInitNewFeatures(new.feat.str)
	# stats
	#printf("bInitNewFeatures took :%f secs", time.taken[[3]])
	init.time <- init.time + time.taken[[3]]
	# stats
	time.taken <- bInitDecodeIndices(str.decode.indices)
	#printf("bInitDecodeIndices took :%f secs", time.taken[[3]])
	# stats
	init.time <- init.time + time.taken[[3]]
	# stats
	
	# DL calls end#
	
	#stats
	time.taken <- system.time(table.name <- GetRelationName(decode.indices, dataset.id,
batch.tables.map), gcFirst = TRUE)
	if(time.taken[[3]] < 1)
		time.taken[[3]] <- 0
	UpdateStatsList('mat.time', time.taken[[3]])	

	#stats
	decode.indices.map <- GetDecodeIndicesMap(table.name, dataset.id, sort(decode.indices))
	str.decode.indices.map <- paste(decode.indices.map, collapse = ",")
	y.idx <- GetTargetIndexFromDerivedDataset(table.name, dataset.id)
	str.y.idx <- paste(y.idx, sep="")
	
	#DL call start#
	decode.indices <- setdiff(decode.indices, GetNewFeatures(dataset.id))
	str.decode.indices <- paste(sort(decode.indices), collapse = ",")
	time.taken <- bInitDecodeIndicesMap(paste(str.decode.indices, str.decode.indices.map, sep="#"))
	# stats
	#printf("bInitDecodeIndicesMap took :%f secs", time.taken[[3]])
	init.time <- init.time + time.taken[[3]]
	# stats

	time.taken <- bSetYIdx(str.y.idx)
	# stats
	init.time <- init.time + time.taken[[3]]
	# stats
	#DL call end#

		
	
	# DL calls start#
	time.taken <- bInitObjectList(obj.list)
	#printf("bInitObjectList took :%f secs", time.taken[[3]])
	# stats
	init.time <- init.time + time.taken[[3]]
	printf("Init time  :%f secs", time.taken[[3]])
	UpdateStatsList('init.time', init.time)
	# stats
	#browser()
	# update the number of times the data gets accessed
	UpdateNumTimesAccessed(table.name)
	time.taken <- bTblIterate(table.name)
	printf("Execution time  :%f secs", time.taken[[3]])
	UpdateStatsList('exe.time', time.taken[[3]])
	# DL calls end#

	retrieve.result <- 0
	for(step.num in step.num.list) {
		node <- node.list[[step.num]]
		if((node$data.aware == 1) && (node$completed == 0)) {
			if(are.inputs.computed(step.num) == TRUE) {
				obj <- obj.store[[step.num]]
				# DL calls start#
				time.taken <- system.time(res.string <- bResultInStages(obj))
				# stats
				retrieve.result <- retrieve.result + time.taken[[3]]
				# stats
				# DL calls end#
				ConsumeResult(step.num, res.string)
				node.list[[step.num]]$completed = 1
			}
		}
	}	
	printf("Retrieve result time  :%f secs", time.taken[[3]])
	UpdateStatsList('retrieve.result', retrieve.result)
	return(node.list)
	
}

RunDataAgnosticOperation <- function(node.list,  dataset.id) {
	
	steps.to.consider <- GetStepsForDataset(dataset.id)
	
	if(length(steps.to.consider) == 0) {
		stop('There are no qualifying steps for the dataset id')
	}
	
	for(step.num in steps.to.consider) {
		node <- node.list[[step.num]]
		if((node$data.aware == 0) && (node$completed == 0)) {
			if(are.inputs.computed(step.num)) { 
				ExeAgnosticOperation(step.num)
				node$completed = 1		
			}
		} 
	}
	return(node.list)
}


