
#Copyright 2013 Pradap  and Arun Kumar and Christopher Re
#
#Licensed under the Apache License, Version 2.0 (the "License");
#you may not use this file except in compliance with the License.
#You may obtain a copy of the License at
#
#http://www.apache.org/licenses/LICENSE-2.0
#
#Unless required by applicable law or agreed to in writing, software
#distributed under the License is distributed on an "AS IS" BASIS,
#WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#See the License for the specific language governing permissions and
#limitations under the License.
#
#
StepAdd <- function(inp.set, mask.set, train.type, eval.type, dataset.id, num.iters = 2, step.size = 0.01, decay = 1, init.wt = 0, num.folds = 1) {
	
	# Error handling
	if(is.null(inp.set)) {
		stop('Argument inp.set cannot be NULL')
	}
	if(is.null(mask.set)) {
		stop('Argument mask.set cannot be NULL')
	}

	train.type <- tolower(train.type)
	eval.type <- tolower(eval.type)

	if(eval.type == "aic") {
		if(num.folds != 1) {
			stop('Argument num.folds must be set to 1 for AIC')
		}
	}


	if(train.type == "igd") {
		res <- SkelStepAddIgdTrain(inp.set, mask.set, num.iters, step.size, decay, init.wt,
num.folds, dataset.id)
	} else if(train.type == "cg") {
		res <- SkelStepAddCgTrain(inp.set, mask.set, num.iters, init.wt, num.folds,
dataset.id)
	}

	res.list <- res$out.list
	
	pgm.ctr <- res$pgm.ctr
	step.num.list <- res$step.num.list


	inp.list.id <- res.list$list.id
	res <- SkelStepAddEvalTest(inp.set, mask.set, eval.type, pgm.ctr, inp.list.id, dataset.id)
	
	# accumulate the step.nums
	res.list <- res$out.list
	step.num.list <- c(step.num.list, res$step.num.list)
	inp.list.id <- res.list$list.id
	pgm.ctr <- res$pgm.ctr

	res <- SkelStepAddBestK(inp.list.id, pgm.ctr, dataset.id)
	step.num.list <- c(step.num.list, res$step.num.list)

	out.set <- res$out.set
	set.id <- out.set$feat.id

	if(is.interactive() == TRUE) {
		RunInteractiveOperation(step.num.list, dataset.id)
	}
	ret.set <- GetFeatureSet(set.id)
	feat.set <- FeatureSetClass(set.id, ret.set)
	if(is.updateprogram()) {
		inp.set.fs <- paste0("fs", inp.set$feat.id)
		mask.set.fs <- paste0("fs", mask.set$feat.id)
		pgm.str <- paste0("fs", feat.set$feat.id, " <- ", "StepAdd(inp.set = ", inp.set.fs, ", mask.set = ", mask.set.fs,", train.type = \"", train.type , "\", eval.type = \"", eval.type ,"\", dataset.id = ", dataset.id, ", num.iters = ", num.iters, ", step.size = ", step.size,", init.wt = ", init.wt ,", decay = ", decay , ", num.folds = ", num.folds, ")")
		AddProgramLine(pgm.str)

	}
	return(feat.set)

}

StepDel <- function(inp.set, train.type, eval.type, dataset.id, num.iters = 2, step.size = 0.01,
decay = 1, init.wt = 0, num.folds = 1) {
	
	# Error handling
	if(is.null(inp.set)) {
		stop('Argument inp.set cannot be NULL')
	}
	
	train.type <- tolower(train.type)
	eval.type <- tolower(eval.type)

	if(eval.type == "aic") {
		if(num.folds != 1) {
			stop('Argument num.folds must be set to 1 for AIC')
		}
	}

	if(train.type == "igd") {
		res <- SkelStepDelIgdTrain(inp.set, num.iters, step.size, decay, init.wt, num.folds,
dataset.id)
	} else if(train.type == "cg") {
		res <- SkelStepDelCgTrain(inp.set, num.iters, init.wt, num.folds,
dataset.id)
	}

	res.list <- res$out.list
	pgm.ctr <- res$pgm.ctr
	step.num.list <- res$step.num.list 

	inp.list.id <- res.list$list.id
	res <- SkelStepDelEvalTest(inp.set, eval.type, pgm.ctr, inp.list.id, dataset.id)
	
	# accumulate the step.nums
	res.list <- res$out.list
	step.num.list <- c(step.num.list, res$step.num.list)
	inp.list.id <- res.list$list.id
	pgm.ctr <- res$pgm.ctr
	
	res <- SkelStepDelBestK(inp.list.id, pgm.ctr, dataset.id)
	step.num.list <- c(step.num.list, res$step.num.list)
	

	out.set <- res$out.set
	set.id <- out.set$feat.id

	if(is.interactive() == TRUE) {
		RunInteractiveOperation(step.num.list, dataset.id)
	}
	
	ret.set <- GetFeatureSet(set.id)
	feat.set <- FeatureSetClass(set.id, ret.set)
	if(is.updateprogram()) {
		inp.set.fs <- paste0("fs", inp.set$feat.id)
		pgm.str <- paste0("fs", feat.set$feat.id, " <- ", "StepDel(inp.set = ", inp.set.fs, ", train.type = \"", train.type , "\", eval.type = \"", eval.type ,"\", dataset.id = ", dataset.id, ", num.iters = ", num.iters, ", step.size = ", step.size,", init.wt = ", init.wt ,", decay = ", decay , ", num.folds = ", num.folds, ")")
		AddProgramLine(pgm.str)

	}
	return(feat.set)
	#return(ret.set)
}
