
public class MyEvalLRCVTest extends MyAggregate{
    private MyFeatureSubsetLRCV objFS[];
    private int numFolds;

    public int initialize(String s) {
        // should initialize feature subsets and the coef values.
        // schema numfolds#fs1#fs2#fs3
        String []splitted = s.split("#");
        numFolds = Integer.parseInt(splitted[0]);
        objFS = new MyFeatureSubsetLRCV[splitted.length - 1]; // because the first one was the fold number
        for(int i = 1; i < splitted.length; i ++) {
            objFS[i-1]= new MyFeatureSubsetLRCV();
            objFS[i-1].initFeatureSubset(numFolds, splitted[i]); // hardcoding the number of folds to 1.
        }
        return 1;
    }
    private int test(int foldNum) {
        for(int i = 0; i < objFS.length; i ++) {
            int numFeatures = objFS[i].featureIndices.length;
            double wx = 0.0;
            for(int j = 0; j < numFeatures; j ++ ) {
                int featureIndex = objFS[i].featureIndices[j];
                double wVal = objFS[i].coefficients[foldNum][j]; //fold is hardcoded
                double x = MyTuple.xVal[featureIndex];
                wx += wVal*x;
            }
            double dPredict = 1.0/(1.0+Math.exp(-1.0*wx));
            int yVal = MyTuple.yVal > 0.5? 1: -1;
            int label = dPredict > 0.5 ? 1 : -1;
            objFS[i].numObservations[foldNum] ++;
            if(yVal == label)
                objFS[i].numSuccess[foldNum] ++;
            else
                objFS[i].numFailures[foldNum] ++;
        }
        return 1;
    }
    public int iterate() {
	if(numFolds == 1) {
		test(0);
	}
        else {
		for(int i = 0; i < numFolds; i ++) {
	            if( ((MyTuple.recId % numFolds) + 1) == (i + 1)) {
        	        test(i);
            		}
        	}
	}
        return 1;
    }
    public String result() {
        // layout : fs1#fs2#fs3...#fsn
        // fs1 : featuresetIndices $ fold1 ~ fold2 ~ .. foldn
        // fold"k" = cv
        StringBuilder sb = new StringBuilder();
        String res;
        for(int i = 0; i < objFS.length; i ++)
        {
            res = objFS[i].getCVResult();
            sb.append(res);
            if(i != objFS.length - 1)
            {
                sb.append("#");
            }

        }
        return sb.toString();
    }
    public int terminate() {
        objFS = null;
        return 1;
    }



}
