import java.math.BigInteger;
import java.security.SecureRandom;
import java.util.ArrayList;
import java.util.HashMap;

public class MyUDFManager {
    private static HashMap objMap;
    private static SecureRandom sRndm;
    private static ArrayList objList;
    private static int matSize;
    private static int sndFSMflag;
    private static StringBuilder inpString;
    private static String[] resString;
    
    public static int init() {
        objMap = new HashMap();
        inpString = null;
        resString = null;
        sndFSMflag = -1;
        MyTuple.recId = 0;
        return 1;
    }
    public static int terminate(){
        objMap = null;
        sRndm = null;
        objList = null;
        inpString = null;
        resString = null;
        sndFSMflag = -1;
        return 1;
    }
    private static String nextSessionId() {
        sRndm = new SecureRandom();
        return(new BigInteger(130,sRndm).toString());
    }
    public static int initNumAttributes(String s) {
        // initialize the number of features.
        // totNumFeatures,origNumFeatures, yIdx
        String splitted[] = s.split(",");
	SimpleLogger.write("InitAttr (s) : " + s);
        MyTuple.initAttrVector(Integer.parseInt(splitted[0]), Integer.parseInt(splitted[1]), Integer.parseInt(splitted[2]));
        init();
        return 1;
    }
    public static int initNewFeatures(String s)
    {
        MyTuple.initNewFeatures(s);
        return 1;
    }
    public static int initDecodeIndices(String s) {
        // delegate to MyTuple
        MyTuple.setDecodeIndices(s);
        return 1;
    }
    public static int initDecodeIndicesMap(String s) {
        MyTuple.setDecodeIndicesMap(s);
        return 1;
    }
    public static int setYIdx(String s) {
        MyTuple.setYIdx(s);
        return 1;
    }
    public static int initialize(String s) {
        String  splitted[] = s.split("@");
        String uId = splitted[0];
        if (!objMap.containsKey(uId)) {
            return 0;
        } else {
            MyObjectInfo obj = (MyObjectInfo)objMap.get(uId);
            MyAggregate aObj = obj.getObj();
            aObj.initialize(splitted[1]);
            return 1;
        }
    }
    public static String result(String s) {
        if (!objMap.containsKey(s)) {
            return null;
        } else {
            MyObjectInfo obj = (MyObjectInfo)objMap.get(s);
            MyAggregate aObj = obj.getObj();
            return(aObj.result());
        }
    }
    public static int deleteObj(String s) {
        if (!objMap.containsKey(s)) {
            return 0;
        } else {
	    MyObjectInfo obj = (MyObjectInfo)objMap.remove(s);
            MyAggregate aObj = obj.getObj();
            return(aObj.terminate());
        }
    }
    public static int initObjectList(String s) {
        objList = new ArrayList();
        int flag = 0;

        String splitted[] = s.split("@");
        for (String uid : splitted) {
            if (!objMap.containsKey(uid)) {
                flag = 1;
            } else {
                // found the object
                MyObjectInfo obj = (MyObjectInfo) objMap.get(uid);
                MyAggregate aObj = obj.getObj();
                objList.add(aObj);
            }

        }
        return flag;
    }
    public static int iterate(byte[] b){
        MyTuple.decode(b);
        for (Object anObjList : objList) {
            MyAggregate aObj = (MyAggregate) anObjList;
            aObj.iterate();
        }
        //MyTuple.print();
        return 1;
    }
    public static int iterate(){
        MyTuple.recId ++;
	//SimpleLogger.write("Inside Iterate()");
        for (Object anObjList : objList) {
	//    SimpleLogger.write("Inside objList loop");
            MyAggregate aObj = (MyAggregate) anObjList;
            aObj.iterate();
        }
	//SimpleLogger.write("Returning..");
        //MyTuple.print();
        return 1;
    }
    public static String createObj(String s) {
        MyObjectInfo obj = new MyObjectInfo(s);
        String uId = null;
        while(true) {
            uId = nextSessionId();
            if (objMap.containsKey(uId)) {
                continue;
            } else {
                objMap.put(uId, obj) ;
                break;
            }
        }
        return uId;
    }


    /** Test functions */
    public static int nullUDF(byte[] b) {
        return 1;
    }
    public static int noDataFunction() {
        return 1;
    }
    public static int nullAndParse(byte[] b) {
	
	//SimpleLogger.write(b.length + "") ;
	//SimpleLogger.write("+++");
        MyTuple.decode(b);
	//MyTuple.print();
        return 1;
    }
    /** Utility functions */
    public static int initMatSize(String s) {
	matSize = Integer.parseInt(s);
	return 1;
    }
    public static byte[] materialize(byte[] b) {
        byte[] nb = new byte[(matSize*8)];
        for(int i = 0; i < (8*matSize); i++) {
            nb[i] = b[i];

        }
        return nb;
    }

    public static int getSndFSMflag() {
        return sndFSMflag;
    }

    public static void setSndFSMflag(int sndFSMflag) {
        MyUDFManager.sndFSMflag = sndFSMflag;
    }
    public static String getResultInStages(String s)
    {
        if(getSndFSMflag() == -1 )
        {
            //first time this is getting called.
            String res = result(s);
            int len = 3000;
            //int len = 10;

            //split it into Strings.
            resString = new String[(int) Math.ceil((double) res.length() / (double) len)];
            for(int i = 0; i < resString.length; i ++ )
            {
                resString[i]  = res.substring(i*len, Math.min(res.length(), (i+1)*len));
            }
        }
        // There are more strings to be sent
        int curIndex = getSndFSMflag();
        curIndex += 1;
        setSndFSMflag(curIndex);
        StringBuilder retRes = new StringBuilder();
        if(getSndFSMflag() == resString.length - 1)
        {
            // last chunk.
            retRes.append("END%").append(resString[getSndFSMflag()]);
            setSndFSMflag(-1);
        }
        else
        {
            retRes.append("MRE%").append(resString[getSndFSMflag()]);
        }
        return(retRes.toString());
    }
    public static int initializeInStages(String s)
    {
        // protocol:
        //  STM%  : allocate memory for input string and append.
        //  STE%  : Single initialization String.
        //  MRE%  : append
        //  END%  : append and call initialize
        String[] in = s.split("%");
        if(in[0].equalsIgnoreCase("STM"))
        {
            inpString = new StringBuilder();
            inpString.append(in[1]);
        }
        else if(in[0].equalsIgnoreCase("STE"))
        {
            inpString = new StringBuilder();
            inpString.append(in[1]);
            initialize(inpString.toString());
            inpString = null;
        }
        else if(in[0].equalsIgnoreCase("MRE"))
        {
            inpString.append(in[1]);
        }
        else if(in[0].equalsIgnoreCase("END"))
        {
            inpString.append(in[1]);
            initialize(inpString.toString());
            inpString = null;
        }
	return 1;
    }

}
